@@ -6,7 +6,7 @@
 ---- importing ----
 if string.sub(_VERSION,1,7)=='Lua 5.0' then
 	-- lua5.0 doesnt have a nice way to do this
-	lib=loadlib('example.dll','Example_Init') or loadlib('example.so','Example_Init')
+	lib=loadlib('example.dll','luaopen_example') or loadlib('example.so','luaopen_example')
 	assert(lib)()
 else
 	-- lua 5.1 does
