@@ -1,13 +1,17 @@
 /* ----------------------------------------------------------------------------- 
- * See the LICENSE file for information on copyright, usage and redistribution
- * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
  *
  * contract.cxx
  *
  * Support for Wrap by Contract in SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_contract_cxx[] = "$Id: contract.cxx 10003 2007-10-17 21:42:11Z wsfulton $";
+char cvsroot_contract_cxx[] = "$Id: contract.cxx 12830 2011-10-30 21:51:50Z wsfulton $";
 
 #include "swigmod.h"
 
@@ -46,6 +50,7 @@ public:
   int extendDirective(Node *n);
   int importDirective(Node *n);
   int includeDirective(Node *n);
+  int namespaceDeclaration(Node *n);
   int classDeclaration(Node *n);
   virtual int top(Node *n);
 };
@@ -208,7 +213,7 @@ String *Contracts::make_expression(String *s, Node *n) {
   for (ei = First(list_assert); ei.item; ei = Next(ei)) {
     expr = ei.item;
     if (Len(expr)) {
-      Replaceid(expr, Getattr(n, "name"), "result");
+      Replaceid(expr, Getattr(n, "name"), Swig_cresult_name());
       if (Len(str_assert))
 	Append(str_assert, "&&");
       Printf(str_assert, "(%s)", expr);
@@ -220,14 +225,14 @@ String *Contracts::make_expression(String *s, Node *n) {
 
 /* This function substitutes parameter names for argument names in the
    contract specification.  Note: it is assumed that the wrapper code 
-   uses arg1--argn for arguments. */
+   uses arg1 for self and arg2..argn for arguments. */
 
 void Contracts::substitute_parms(String *s, ParmList *p, int method) {
   int argnum = 1;
   char argname[32];
 
   if (method) {
-    Replaceid(s, "self", "arg0");
+    Replaceid(s, "$self", "arg1");
     argnum++;
   }
   while (p) {
@@ -320,16 +325,23 @@ int Contracts::constructorDeclaration(Node *n) {
 int Contracts::externDeclaration(Node *n) {
   return emit_children(n);
 }
+
 int Contracts::extendDirective(Node *n) {
   return emit_children(n);
 }
+
 int Contracts::importDirective(Node *n) {
   return emit_children(n);
 }
+
 int Contracts::includeDirective(Node *n) {
   return emit_children(n);
 }
 
+int Contracts::namespaceDeclaration(Node *n) {
+  return emit_children(n);
+}
+
 int Contracts::classDeclaration(Node *n) {
   int ret = SWIG_OK;
   InClass = 1;
