@@ -40,11 +40,11 @@
  * ver009
    class support: ok for basic types, but methods still TDB
    (code is VERY messed up & needs to be cleaned)
- 
- 
+ * ver010
+   Added support for embedded Lua. Try swig -lua -help for more information
 */
 
-char cvsroot_lua_cxx[] = "$Id: lua.cxx 12655 2011-05-05 06:23:02Z wsfulton $";
+char cvsroot_lua_cxx[] = "$Id: lua.cxx 13119 2012-05-24 23:05:07Z wsfulton $";
 
 #include "swigmod.h"
 
@@ -83,10 +83,15 @@ void display_mapping(DOH *d) {
 NEW LANGUAGE NOTE:END ************************************************/
 static const char *usage = (char *) "\
 Lua Options (available with -lua)\n\
-     [no additional options]\n\
+     -elua           - Generates LTR compatible wrappers for smaller devices running elua\n\
+     -eluac          - LTR compatible wrappers in \"crass compress\" mode for elua\n\
+     -nomoduleglobal - Do not register the module name as a global variable \n\
+                       but return the module table from calls to require.\n\
 \n";
 
-
+static int nomoduleglobal = 0;
+static int elua_ltr = 0;
+static int eluac_ltr = 0;
 
 /* NEW LANGUAGE NOTE:***********************************************
  To add a new language, you need to derive your class from
@@ -110,6 +115,9 @@ private:
   String *s_methods_tab;	// table of class methods
   String *s_attr_tab;		// table of class atributes
   String *s_luacode;		// luacode to be called during init
+  String *s_dot_get;            // table of variable 'get' functions
+  String *s_dot_set;            // table of variable 'set' functions
+  String *s_vars_meta_tab;      // metatable for variables
 
   int have_constructor;
   int have_destructor;
@@ -172,6 +180,15 @@ public:
       if (argv[i]) {
         if (strcmp(argv[i], "-help") == 0) {	// usage flags
           fputs(usage, stdout);
+        } else if (strcmp(argv[i], "-nomoduleglobal") == 0) {
+          nomoduleglobal = 1;
+          Swig_mark_arg(i);
+        } else if(strcmp(argv[i], "-elua") == 0) {
+          elua_ltr = 1;
+          Swig_mark_arg(i);
+        } else if(strcmp(argv[i], "-eluac") == 0) {
+          eluac_ltr = 1;
+          Swig_mark_arg(i);
         }
       }
     }
@@ -251,6 +268,10 @@ public:
     s_var_tab = NewString("");
     //    s_methods_tab    = NewString("");
     s_const_tab = NewString("");
+
+    s_dot_get = NewString("");
+    s_dot_set = NewString("");
+    s_vars_meta_tab = NewString("");
     
     s_luacode = NewString("");
     Swig_register_filebyname("luacode", s_luacode);
@@ -263,6 +284,14 @@ public:
     Printf(f_runtime, "\n");
     Printf(f_runtime, "#define SWIGLUA\n");
 
+    emitLuaFlavor(f_runtime);
+
+    if (nomoduleglobal) {
+      Printf(f_runtime, "#define SWIG_LUA_NO_MODULE_GLOBAL\n");
+    } else {
+      Printf(f_runtime, "#define SWIG_LUA_MODULE_GLOBAL\n");
+    }
+
     //    if (NoInclude) {
     //      Printf(f_runtime, "#define SWIG_NOINCLUDE\n");
     //    }
@@ -277,12 +306,31 @@ public:
     Printf(f_header, "#define SWIG_name      \"%s\"\n", module);
     Printf(f_header, "#define SWIG_init      luaopen_%s\n", module);
     Printf(f_header, "#define SWIG_init_user luaopen_%s_user\n\n", module);
-    Printf(f_header, "#define SWIG_LUACODE   luaopen_%s_luacode\n\n", module);
+    Printf(f_header, "#define SWIG_LUACODE   luaopen_%s_luacode\n", module);
 
-    Printf(s_cmd_tab, "\nstatic const struct luaL_reg swig_commands[] = {\n");
-    Printf(s_var_tab, "\nstatic swig_lua_var_info swig_variables[] = {\n");
-    Printf(s_const_tab, "\nstatic swig_lua_const_info swig_constants[] = {\n");
-    Printf(f_wrappers, "#ifdef __cplusplus\nextern \"C\" {\n#endif\n");
+    if (elua_ltr || eluac_ltr)
+      Printf(f_header, "#define swig_commands  %s_map\n\n", module);
+
+    if (elua_ltr || eluac_ltr) {
+      Printf(s_cmd_tab, "\n#define MIN_OPT_LEVEL 2\n#include \"lrodefs.h\"\n");
+      Printf(s_cmd_tab, "#include \"lrotable.h\"\n");
+      Printf(s_cmd_tab, "\nconst LUA_REG_TYPE swig_constants[];\n");
+      if (elua_ltr)
+        Printf(s_cmd_tab, "const LUA_REG_TYPE mt[];\n");
+
+      Printf(s_cmd_tab, "\nconst LUA_REG_TYPE swig_commands[] = {\n");
+      Printf(s_const_tab, "\nconst LUA_REG_TYPE swig_constants[] = {\n");
+      Printf(f_wrappers, "#ifdef __cplusplus\nextern \"C\" {\n#endif\n");
+      if (elua_ltr) {
+        Printf(s_dot_get, "\nconst LUA_REG_TYPE dot_get[] = {\n");
+        Printf(s_dot_set, "\nconst LUA_REG_TYPE dot_set[] = {\n");
+      }
+    } else {
+      Printf(s_cmd_tab, "\nstatic const struct luaL_Reg swig_commands[] = {\n");
+      Printf(s_var_tab, "\nstatic swig_lua_var_info swig_variables[] = {\n");
+      Printf(s_const_tab, "\nstatic swig_lua_const_info swig_constants[] = {\n");
+      Printf(f_wrappers, "#ifdef __cplusplus\nextern \"C\" {\n#endif\n");
+    }
 
     /* %init code inclusion, effectively in the SWIG_init function */
     Printf(f_init, "void SWIG_init_user(lua_State* L)\n{\n");
@@ -293,11 +341,39 @@ public:
     Printf(f_wrappers, "#ifdef __cplusplus\n}\n#endif\n");
 
     // Done.  Close up the module & write to the wrappers
-    Printv(s_cmd_tab, tab4, "{0,0}\n", "};\n", NIL);
-    Printv(s_var_tab, tab4, "{0,0,0}\n", "};\n", NIL);
-    Printv(s_const_tab, tab4, "{0,0,0,0,0,0}\n", "};\n", NIL);
-    Printv(f_wrappers, s_cmd_tab, s_var_tab, s_const_tab, NIL);
-    SwigType_emit_type_table(f_runtime, f_wrappers);
+    if (elua_ltr || eluac_ltr) {
+      Printv(s_cmd_tab, tab4, "{LSTRKEY(\"const\"), LROVAL(swig_constants)},\n", NIL);
+      if (elua_ltr)
+        Printv(s_cmd_tab, tab4, "{LSTRKEY(\"__metatable\"), LROVAL(mt)},\n", NIL);
+      Printv(s_cmd_tab, tab4, "{LNILKEY, LNILVAL}\n", "};\n", NIL);
+      Printv(s_const_tab, tab4, "{LNILKEY, LNILVAL}\n", "};\n", NIL);
+    } else {
+      Printv(s_cmd_tab, tab4, "{0,0}\n", "};\n", NIL);
+      Printv(s_var_tab, tab4, "{0,0,0}\n", "};\n", NIL);
+      Printv(s_const_tab, tab4, "{0,0,0,0,0,0}\n", "};\n", NIL);
+    }
+
+    if (elua_ltr) {
+      /* Generate the metatable */
+      Printf(s_vars_meta_tab, "\nconst LUA_REG_TYPE mt[] = {\n");
+      Printv(s_vars_meta_tab, tab4, "{LSTRKEY(\"__index\"), LFUNCVAL(SWIG_Lua_module_get)},\n", NIL);
+      Printv(s_vars_meta_tab, tab4, "{LSTRKEY(\"__newindex\"), LFUNCVAL(SWIG_Lua_module_set)},\n", NIL);
+      Printv(s_vars_meta_tab, tab4, "{LSTRKEY(\".get\"), LROVAL(dot_get)},\n", NIL);
+      Printv(s_vars_meta_tab, tab4, "{LSTRKEY(\".set\"), LROVAL(dot_set)},\n", NIL);
+      Printv(s_vars_meta_tab, tab4, "{LNILKEY, LNILVAL}\n};\n", NIL);
+
+      Printv(s_dot_get, tab4, "{LNILKEY, LNILVAL}\n};\n", NIL);
+      Printv(s_dot_set, tab4, "{LNILKEY, LNILVAL}\n};\n", NIL);
+    }
+
+    if (elua_ltr || eluac_ltr) {
+      /* Final close up of wrappers */
+      Printv(f_wrappers, s_cmd_tab, s_dot_get, s_dot_set, s_vars_meta_tab, s_var_tab, s_const_tab, NIL);
+      SwigType_emit_type_table(f_runtime, f_wrappers);
+    } else {
+      Printv(f_wrappers, s_cmd_tab, s_var_tab, s_const_tab, NIL);
+      SwigType_emit_type_table(f_runtime, f_wrappers);
+    }
 
     /* NEW LANGUAGE NOTE:***********************************************
      this basically combines several of the strings together
@@ -323,6 +399,9 @@ public:
     Close(f_begin);
     Delete(f_runtime);
     Delete(f_begin);
+    Delete(s_dot_get);
+    Delete(s_dot_set);
+    Delete(s_vars_meta_tab);
 
     /* Done */
     return SWIG_OK;
@@ -553,7 +632,7 @@ public:
         //        }
         //        else returnval++;
         Replaceall(tm, "$source", Getattr(p, "lname"));
-        Replaceall(tm, "$target", "result");
+        Replaceall(tm, "$target", Swig_cresult_name());
         Replaceall(tm, "$arg", Getattr(p, "emit:input"));
         Replaceall(tm, "$input", Getattr(p, "emit:input"));
         Printv(outarg, tm, "\n", NIL);
@@ -574,7 +653,7 @@ public:
     this is because there is a typemap for void
     NEW LANGUAGE NOTE:END ************************************************/
     // Return value if necessary
-    if ((tm = Swig_typemap_lookup_out("out", n, "result", f, actioncode))) {
+    if ((tm = Swig_typemap_lookup_out("out", n, Swig_cresult_name(), f, actioncode))) {
       // managing the number of returning variables
       //      if (numoutputs=Getattr(tm,"numoutputs")){
       //              int i=GetInt(tm,"numoutputs");
@@ -582,7 +661,7 @@ public:
       //              returnval+=GetInt(tm,"numoutputs");
       //      }
       //        else returnval++;
-      Replaceall(tm, "$source", "result");
+      Replaceall(tm, "$source", Swig_cresult_name());
       if (GetFlag(n, "feature:new")) {
         Replaceall(tm, "$owner", "1");
       } else {
@@ -603,15 +682,15 @@ public:
 
     /* Look to see if there is any newfree cleanup code */
     if (GetFlag(n, "feature:new")) {
-      if ((tm = Swig_typemap_lookup("newfree", n, "result", 0))) {
-        Replaceall(tm, "$source", "result");
+      if ((tm = Swig_typemap_lookup("newfree", n, Swig_cresult_name(), 0))) {
+        Replaceall(tm, "$source", Swig_cresult_name());
         Printf(f->code, "%s\n", tm);
       }
     }
 
     /* See if there is any return cleanup code */
-    if ((tm = Swig_typemap_lookup("ret", n, "result", 0))) {
-      Replaceall(tm, "$source", "result");
+    if ((tm = Swig_typemap_lookup("ret", n, Swig_cresult_name(), 0))) {
+      Replaceall(tm, "$source", Swig_cresult_name());
       Printf(f->code, "%s\n", tm);
     }
 
@@ -630,7 +709,7 @@ public:
 
     /* Substitute the function name */
     Replaceall(f->code, "$symname", iname);
-    Replaceall(f->code, "$result", "result");
+    Replaceall(f->code, "$result", Swig_cresult_name());
 
     /* Dump the function out */
     /* in Lua we will not emit the destructor as a wrappered function,
@@ -651,9 +730,13 @@ public:
     /* Now register the function with the interpreter. */
     if (!Getattr(n, "sym:overloaded")) {
       //      add_method(n, iname, wname, description);
-      if (current==NO_CPP || current==STATIC_FUNC) // emit normal fns & static fns
-        Printv(s_cmd_tab, tab4, "{ \"", iname, "\", ", Swig_name_wrapper(iname), "},\n", NIL);
+      if (current==NO_CPP || current==STATIC_FUNC) { // emit normal fns & static fns
+        if(elua_ltr || eluac_ltr)
+          Printv(s_cmd_tab, tab4, "{LSTRKEY(\"", iname, "\")", ", LFUNCVAL(", Swig_name_wrapper(iname), ")", "},\n", NIL);
+        else
+          Printv(s_cmd_tab, tab4, "{ \"", iname, "\", ", Swig_name_wrapper(iname), "},\n", NIL);
       //      Printv(s_cmd_tab, tab4, "{ SWIG_prefix \"", iname, "\", (swig_wrapper_func) ", Swig_name_wrapper(iname), "},\n", NIL);
+      }
     } else {
       if (!Getattr(n, "sym:nextSibling")) {
         dispatchFunction(n);
@@ -762,7 +845,7 @@ public:
     String *getName = Swig_name_wrapper(Swig_name_get(NSPACE_TODO, iname));
     String *setName = 0;
     // checking whether it can be set to or not appears to be a very error prone issue
-    // I refered to the Language::variableWrapper() to find this out
+    // I referred to the Language::variableWrapper() to find this out
     bool assignable=is_assignable(n) ? true : false;
     SwigType *type = Getattr(n, "type");
     String *tm = Swig_typemap_lookup("globalin", n, iname, 0);
@@ -777,8 +860,17 @@ public:
       setName = NewString("SWIG_Lua_set_immutable"); // error message
       //setName = NewString("0");
     }
+
     // register the variable
-    Printf(s_var_tab, "%s{ \"%s\", %s, %s },\n", tab4, iname, getName, setName);
+    if (elua_ltr) {
+      Printf(s_dot_get, "%s{LSTRKEY(\"%s\"), LFUNCVAL(%s)},\n", tab4, iname, getName);
+      Printf(s_dot_set, "%s{LSTRKEY(\"%s\"), LFUNCVAL(%s)},\n", tab4, iname, setName);
+    } else if (eluac_ltr) {
+      Printv(s_cmd_tab, tab4, "{LSTRKEY(\"", iname, "_get", "\")", ", LFUNCVAL(", getName, ")", "},\n", NIL);
+      Printv(s_cmd_tab, tab4, "{LSTRKEY(\"", iname, "_set", "\")", ", LFUNCVAL(", setName, ")", "},\n", NIL);
+    } else {
+      Printf(s_var_tab, "%s{ \"%s\", %s, %s },\n", tab4, iname, getName, setName);
+    }
     Delete(getName);
     Delete(setName);
     return result;
@@ -812,7 +904,7 @@ public:
       Replaceall(tm, "$target", name);
       Replaceall(tm, "$value", value);
       Replaceall(tm, "$nsname", nsname);
-      Printf(s_const_tab, "%s,\n", tm);
+      Printf(s_const_tab, "    %s,\n", tm);
     } else if ((tm = Swig_typemap_lookup("constcode", n, name, 0))) {
       Replaceall(tm, "$source", value);
       Replaceall(tm, "$target", name);
@@ -890,7 +982,7 @@ public:
     real_classname = Getattr(n, "name");
     mangled_classname = Swig_name_mangle(real_classname);
 
-    // not sure exactly how this workswhat this works,
+    // not sure exactly how this works,
     // but tcl has a static hashtable of all classes emitted and then only emits code for them once.
     // this fixes issues in test suites: template_default2 & template_specialization
 
@@ -952,7 +1044,7 @@ public:
     Delete(s_attr_tab);
 
     // Handle inheritance
-    // note: with the idea of class hireachied spread over multiple modules
+    // note: with the idea of class hierarchies spread over multiple modules
     // cf test-suite: imports.i
     // it is not possible to just add the pointers to the base classes to the code
     // (as sometimes these classes are not present)
@@ -996,7 +1088,17 @@ public:
     Printv(f_wrappers, "static swig_lua_class _wrap_class_", mangled_classname, " = { \"", class_name, "\", &SWIGTYPE", SwigType_manglestr(t), ",", NIL);
 
     if (have_constructor) {
-      Printf(f_wrappers, "%s", Swig_name_wrapper(Swig_name_construct(NSPACE_TODO, constructor_name)));
+      if (elua_ltr) {
+        Printf(s_cmd_tab, "    {LSTRKEY(\"%s\"), LFUNCVAL(%s)},\n", class_name, \
+        Swig_name_wrapper(Swig_name_construct(NSPACE_TODO, constructor_name)));
+        Printf(f_wrappers, "%s", Swig_name_wrapper(Swig_name_construct(NSPACE_TODO, constructor_name)));
+      } else if (eluac_ltr) {
+        Printv(s_cmd_tab, tab4, "{LSTRKEY(\"", "new_", class_name, "\")", ", LFUNCVAL(", \
+        Swig_name_wrapper(Swig_name_construct(NSPACE_TODO, constructor_name)), ")", "},\n", NIL);
+        Printf(f_wrappers, "%s", Swig_name_wrapper(Swig_name_construct(NSPACE_TODO, constructor_name)));
+      } else {
+        Printf(f_wrappers, "%s", Swig_name_wrapper(Swig_name_construct(NSPACE_TODO, constructor_name)));
+      }
       Delete(constructor_name);
       constructor_name = 0;
     } else {
@@ -1004,7 +1106,12 @@ public:
     }
 
     if (have_destructor) {
-      Printv(f_wrappers, ", swig_delete_", class_name, NIL);
+      if (eluac_ltr) {
+        Printv(s_cmd_tab, tab4, "{LSTRKEY(\"", "free_", class_name, "\")", ", LFUNCVAL(", "swig_delete_", class_name, ")", "},\n", NIL);
+        Printv(f_wrappers, ", swig_delete_", class_name, NIL);
+      } else {
+         Printv(f_wrappers, ", swig_delete_", class_name, NIL);
+      }
     } else {
       Printf(f_wrappers, ",0");
     }
@@ -1070,6 +1177,12 @@ public:
       sname = NewString("SWIG_Lua_set_immutable"); // error message
     }
     Printf(s_attr_tab,"%s{ \"%s\", %s, %s},\n",tab4,symname,gname,sname);
+    if (eluac_ltr) {
+      Printv(s_cmd_tab, tab4, "{LSTRKEY(\"", class_name, "_", symname, "_get", "\")", \
+      ", LFUNCVAL(", gname, ")", "},\n", NIL);
+      Printv(s_cmd_tab, tab4, "{LSTRKEY(\"", class_name, "_", symname, "_set", "\")", \
+      ", LFUNCVAL(", sname, ")", "},\n", NIL);
+    }
     Delete(gname);
     Delete(sname);
     return SWIG_OK;
@@ -1150,9 +1263,10 @@ public:
    */
   String *runtimeCode() {
     String *s = NewString("");
-    const char *filenames[] = { "luarun.swg", 0
-                              }
-                              ;	// must be 0 termiated
+    const char *filenames[] = { "luarun.swg", 0 } ; // must be 0 terminated
+
+    emitLuaFlavor(s);
+
     String *sfile;
     for (int i = 0; filenames[i] != 0; i++) {
       sfile = Swig_include_sys(filenames[i]);
@@ -1175,6 +1289,16 @@ public:
    * helpers
    * --------------------------------------------------------------------- */
 
+  void emitLuaFlavor(String *s) {
+    if (elua_ltr)
+      Printf(s, "#define SWIG_LUA_TARGET SWIG_LUA_FLAVOR_ELUA\n");
+    else if (eluac_ltr)
+      Printf(s, "#define SWIG_LUA_TARGET SWIG_LUA_FLAVOR_ELUAC\n");
+    else
+      Printf(s, "#define SWIG_LUA_TARGET SWIG_LUA_FLAVOR_LUA\n");
+  }
+  
+
   /* This is to convert the string of Lua code into a proper string, which can then be
      emitted into the C/C++ code.
      Basically is is a lot of search & replacing of odd sequences
