@@ -11,7 +11,7 @@
  * Perl5 language module for SWIG.
  * ------------------------------------------------------------------------- */
 
-char cvsroot_perl5_cxx[] = "$Id: perl5.cxx 11945 2010-03-13 23:08:02Z wsfulton $";
+char cvsroot_perl5_cxx[] = "$Id: perl5.cxx 12830 2011-10-30 21:51:50Z wsfulton $";
 
 #include "swigmod.h"
 #include "cparse.h"
@@ -21,14 +21,15 @@ static int treduce = SWIG_cparse_template_reduce(0);
 
 static const char *usage = (char *) "\
 Perl5 Options (available with -perl5)\n\
-     -static         - Omit code related to dynamic loading\n\
-     -nopm           - Do not generate the .pm file\n\
-     -proxy          - Create proxy classes\n\
-     -noproxy        - Don't create proxy classes\n\
+     -compat         - Compatibility mode\n\
      -const          - Wrap constants as constants and not variables (implies -proxy)\n\
-     -nocppcast      - Disable C++ casting operators, useful for generating bugs\n\
      -cppcast        - Enable C++ casting operators\n\
-     -compat         - Compatibility mode\n\n";
+     -nocppcast      - Disable C++ casting operators, useful for generating bugs\n\
+     -nopm           - Do not generate the .pm file\n\
+     -noproxy        - Don't create proxy classes\n\
+     -proxy          - Create proxy classes\n\
+     -static         - Omit code related to dynamic loading\n\
+\n";
 
 static int compat = 0;
 
@@ -308,7 +309,7 @@ public:
     if (no_pmfile) {
       f_pm = NewString(0);
     } else {
-      if (pmfile == NULL) {
+      if (!pmfile) {
 	char *m = Char(module) + Len(module);
 	while (m != Char(module)) {
 	  if (*m == ':') {
@@ -729,9 +730,9 @@ public:
     Swig_director_emit_dynamic_cast(n, f);
     String *actioncode = emit_action(n);
 
-    if ((tm = Swig_typemap_lookup_out("out", n, "result", f, actioncode))) {
+    if ((tm = Swig_typemap_lookup_out("out", n, Swig_cresult_name(), f, actioncode))) {
       SwigType *t = Getattr(n, "type");
-      Replaceall(tm, "$source", "result");
+      Replaceall(tm, "$source", Swig_cresult_name());
       Replaceall(tm, "$target", "ST(argvi)");
       Replaceall(tm, "$result", "ST(argvi)");
       if (is_shadow(t)) {
@@ -759,14 +760,14 @@ public:
     Printv(f->code, cleanup, NIL);
 
     if (GetFlag(n, "feature:new")) {
-      if ((tm = Swig_typemap_lookup("newfree", n, "result", 0))) {
-	Replaceall(tm, "$source", "result");
+      if ((tm = Swig_typemap_lookup("newfree", n, Swig_cresult_name(), 0))) {
+	Replaceall(tm, "$source", Swig_cresult_name());
 	Printf(f->code, "%s\n", tm);
       }
     }
 
-    if ((tm = Swig_typemap_lookup("ret", n, "result", 0))) {
-      Replaceall(tm, "$source", "result");
+    if ((tm = Swig_typemap_lookup("ret", n, Swig_cresult_name(), 0))) {
+      Replaceall(tm, "$source", Swig_cresult_name());
       Printf(f->code, "%s\n", tm);
     }
 
@@ -791,7 +792,7 @@ public:
     } else if (!Getattr(n, "sym:nextSibling")) {
       /* Generate overloaded dispatch function */
       int maxargs;
-      String *dispatch = Swig_overload_dispatch_cast(n, "++PL_markstack_ptr; SWIG_CALLXS(%s); return;", &maxargs);
+      String *dispatch = Swig_overload_dispatch_cast(n, "PUSHMARK(MARK); SWIG_CALLXS(%s); return;", &maxargs);
 
       /* Generate a dispatch wrapper for all overloaded functions */
 
