@@ -13,7 +13,7 @@
 
 // TODO: remove remnants of lisptype
 
-char cvsroot_uffi_cxx[] = "$Id: uffi.cxx 12536 2011-03-14 07:22:08Z wsfulton $";
+char cvsroot_uffi_cxx[] = "$Id: uffi.cxx 12830 2011-10-30 21:51:50Z wsfulton $";
 
 #include "swigmod.h"
 
@@ -324,7 +324,7 @@ int UFFI::functionWrapper(Node *n) {
 	 //"  :strings-convert t\n"
 	 //"  :call-direct %s\n"
 	 //"  :optimize-for-space t"
-	 ")\n", get_ffi_type(n, Getattr(n, "type"), "result")
+	 ")\n", get_ffi_type(n, Getattr(n, "type"), Swig_cresult_name())
 	 //,varargs ? "nil"  : "t"
       );
 
