@@ -14,7 +14,7 @@
  * to easily construct yacc-compatible scanners.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_scanner_c[] = "$Id: scanner.c 12227 2010-09-24 22:13:13Z wsfulton $";
+char cvsroot_scanner_c[] = "$Id: scanner.c 12536 2011-03-14 07:22:08Z wsfulton $";
 
 #include "swig.h"
 #include <ctype.h>
@@ -1156,11 +1156,9 @@ void Scanner_skip_line(Scanner * s) {
 int Scanner_skip_balanced(Scanner * s, int startchar, int endchar) {
   char c;
   int num_levels = 1;
-  int l;
   int state = 0;
   char temp[2] = { 0, 0 };
   String *locator = 0;
-  l = s->line;
   temp[0] = (char) startchar;
   Clear(s->text);
   Setfile(s->text, Getfile(s->str));
