@@ -12,7 +12,7 @@
  * parse trees.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_tree_c[] = "$Id: tree.c 11876 2010-02-27 23:53:33Z wsfulton $";
+char cvsroot_tree_c[] = "$Id: tree.c 13093 2012-05-15 19:51:41Z wsfulton $";
 
 #include "swig.h"
 #include <stdarg.h>
@@ -77,9 +77,9 @@ void Swig_print_node(Node *obj) {
     if ((Cmp(k, "nodeType") == 0) || (Cmp(k, "firstChild") == 0) || (Cmp(k, "lastChild") == 0) ||
 	(Cmp(k, "parentNode") == 0) || (Cmp(k, "nextSibling") == 0) || (Cmp(k, "previousSibling") == 0) || (*(Char(k)) == '$')) {
       /* Do nothing */
-    } else if (Cmp(k, "parms") == 0) {
+    } else if (Cmp(k, "parms") == 0 || Cmp(k, "wrap:parms") == 0) {
       print_indent(2);
-      Printf(stdout, "%-12s - %s\n", k, ParmList_protostr(Getattr(obj, k)));
+      Printf(stdout, "%-12s - %s\n", k, ParmList_str_defaultargs(Getattr(obj, k)));
     } else {
       DOH *o;
       char *trunc = "";
@@ -291,10 +291,10 @@ void Swig_require(const char *ns, Node *n, ...) {
     if (view) {
       if (Strcmp(view, ns) != 0) {
 	Setattr(n, NewStringf("%s:view", ns), view);
-	Setattr(n, "view", ns);
+	Setattr(n, "view", NewString(ns));
       }
     } else {
-      Setattr(n, "view", ns);
+      Setattr(n, "view", NewString(ns));
     }
   }
 }
@@ -337,10 +337,10 @@ void Swig_save(const char *ns, Node *n, ...) {
     if (view) {
       if (Strcmp(view, ns) != 0) {
 	Setattr(n, NewStringf("%s:view", ns), view);
-	Setattr(n, "view", ns);
+	Setattr(n, "view", NewString(ns));
       }
     } else {
-      Setattr(n, "view", ns);
+      Setattr(n, "view", NewString(ns));
     }
   }
 }
